﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.SceneManagement;

public class Player : MonoBehaviour, ITargetHost
{
    public GameObject PrefabHead;
    public TargetFollower PrefabBody;

    public float Speed = 0.5f;
    public float RotationSpeed = 1;

    public AudioClip SoundTakeSeed;
    public AudioClip SoundGrowing;
    public AudioClip SoundHurtObstacle;
    public AudioClip SoundHurtPlayer;
    public AudioClip SoundWin;

    [SerializeField]
    int _posX = 0;
    [SerializeField]
    int _posZ = 0;

    [SerializeField]
    int[,] _dirXZ = new int[4, 2] { { 0, 1 }, { 1, 0 }, { 0, -1 }, { -1, 0 } };
    [SerializeField]
    int _dir = 0;

    int _deltaDir = 0;

    PlayerHead _playerHead;

    TargetFollower _endTail;

    int _targetIndex = 0;
    Dictionary<int, Vector3> _memoTargets = new Dictionary<int, Vector3>();

    int _seedPerBody = 10;
    int _nbSeed = 0;

    // Use this for initialization
    void Start()
    {
        Speed = GlobalData.PlayerSpeed();

        _posX = (LevelManager.WidthX / 4) * 2;
        _posZ = (LevelManager.WidthZ / 4) * 2;

        GameObject head = Instantiate<GameObject>(PrefabHead);
        head.transform.parent = transform;
        head.transform.position = LevelManager.WorldPosition(_posX, _posZ);
        _endTail = head.GetComponent<TargetFollower>();
        _endTail.SetTargetHoster(this, 0, 0, 0);

        _playerHead = head.GetComponentInChildren<PlayerHead>();

        CameraFollow cameraFollow = Camera.main.GetComponent<CameraFollow>();
        cameraFollow.Target = head.transform;
    }

    // Update is called once per frame
    void Update()
    {
        if (Input.GetKeyDown(KeyCode.LeftArrow))
        {
            if (_deltaDir == 1)
            {
                _deltaDir = 0;
            }
            else
            {
                _deltaDir = 3;
            }
        }
        else if (Input.GetKeyDown(KeyCode.RightArrow))
        {
            if (_deltaDir == 3)
            {
                _deltaDir = 0;
            }
            else
            {
                _deltaDir = 1;
            }
        }

        _playerHead.TurnAntenna((_deltaDir == 3) ? -1 : _deltaDir);
    }

    void AddEndTail(bool allowCollider)
    {
        GetComponent<AudioSource>().PlayOneShot(SoundGrowing);

        TargetFollower newEndTail = Instantiate<TargetFollower>(PrefabBody);
        newEndTail.name += string.Format("(nbSeed={0})", _nbSeed);
        newEndTail.transform.parent = transform;
        newEndTail.transform.position = _endTail.transform.position;
        newEndTail.transform.rotation = _endTail.transform.rotation;
        newEndTail.SetTargetHoster(this, _endTail.TargetIndex, 2 / Speed, _endTail.TotalDistance);
        if (allowCollider == false)
        {
            foreach (Collider collider in newEndTail.GetComponentsInChildren<Collider>())
            {
                collider.enabled = false;
            }
        }
        _endTail = newEndTail;
    }

    public Vector3 GetTarget(int index)
    {
        Vector3 target;
        if (index == _targetIndex && _isDead == false)
        {
            // If the head has reached the first target, we create a new one.
            CreateNewTarget();
        }

        if (_memoTargets.TryGetValue(index, out target) == false)
        {
            if (_isDead)
            {
                target = _memoTargets[_targetIndex - 1];
            }
            else
            {
                Debug.LogErrorFormat("Player.Target({0}) not found", index);
                target = LevelManager.WorldPosition(0, 0);
            }

        }

        return target;
    }

    void CreateNewTarget()
    {
        int dir = (_dir + _deltaDir) % 4;
        int x = _posX + _dirXZ[dir, 0];
        int z = _posZ + _dirXZ[dir, 1];

        // If we turn, we check for obstacle because we don't want to die by turning, only by hurting on the front.
        if (_deltaDir != 0)
        {
            if (LevelManager.LevelObstacle(x, z))
            {
                Debug.LogFormat("Prevent a collision with obstacle when turning : continue on straight line\n");
                dir = _dir;
            }
            else
            {
                int bridgeDirection = LevelManager.LevelBridge(x, z);
                if (bridgeDirection != 0 && bridgeDirection != dir + 1)
                {
                    Debug.LogFormat("Prevent a collision with a bridge when turning : continue on straight line\n");
                    dir = _dir;
                }
                else
                {
                    _deltaDir = 0;
                }
            }
        }

        // Store the intermediate position to check bridge.
        x = _posX + _dirXZ[dir, 0];
        z = _posZ + _dirXZ[dir, 1];

        _dir = dir;
        _posX += _dirXZ[dir, 0] * 2;
        _posZ += _dirXZ[dir, 1] * 2;

        Vector3 worldPosition = LevelManager.WorldPosition(_posX, _posZ);

        // Check for bridge in the same direction.
        if (LevelManager.LevelBridge(x, z) == dir + 1)
        {
            Debug.LogFormat("Jump Bridge in direction {0}\n", dir);
            worldPosition.y += LevelManager.BridgeY2;
        }

        _memoTargets.Add(_targetIndex++, worldPosition);
        //Debug.LogFormat("Add _memoTargets({0})\n", _targetIndex);
    }

    public float GetSpeed()
    {
        return Speed;
    }

    public float GetRotationSpeed()
    {
        return RotationSpeed;
    }

    public void OnChildrenTriggerEnter(Collider other)
    {
        if (other.name.StartsWith("Seed"))
        {
            GetComponent<AudioSource>().PlayOneShot(SoundTakeSeed);

            int nbSeed = 1;
            if (int.TryParse(other.name.Substring(4), out nbSeed) == false)
            {
                Debug.LogErrorFormat("int.TryParse(\"{0}\") is false\n", other.name.Substring(4));
            }

            int nbBody = _nbSeed / _seedPerBody;
            _nbSeed += nbSeed;
            Debug.LogFormat("{0}/{1} seeds !\n", _nbSeed, LevelManager.NbSeeds * nbSeed);
            Destroy(other.transform.parent.gameObject);

            if ((_nbSeed / _seedPerBody) > nbBody)
            {
                // No collider on the first body to prevent accidental collision with the head.
                AddEndTail(_nbSeed > _seedPerBody * 2);
            }

            if (_nbSeed == (LevelManager.NbSeeds * nbSeed))
            {
                Speed = 0;
                GetComponent<AudioSource>().PlayOneShot(SoundWin);
                Debug.Log("Win !!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
                GlobalData.CurrentLevel++;
                if (GlobalData.MaxUnlockedLevel < GlobalData.CurrentLevel)
                {
                    GlobalData.MaxUnlockedLevel = GlobalData.CurrentLevel;
                }
                PlayerPrefs.SetInt("MaxUnlockedLevel", GlobalData.MaxUnlockedLevel);
                PlayerPrefs.Save();

                StartCoroutine(WaitUntilQuit());
            }
        }
        else
        {
            Debug.LogFormat("OnChildrenTriggerEnter({0} -> {1})\n", other.transform.parent.name, other.gameObject.name);
            Die(other.name);
        }
    }

    public void OnChildrenCollisionEnter(Collision collision)
    {
        Debug.LogFormat("OnChildrenCollisionEnter({0} -> {1})\n", collision.transform.parent.name, collision.gameObject.name);
        Die(collision.gameObject.name);
    }

    bool _isDead = false;
    void Die(string name)
    {
        if (name.StartsWith("Player"))
        {
            GetComponent<AudioSource>().PlayOneShot(SoundHurtPlayer);
        }
        else
        {
            GetComponent<AudioSource>().PlayOneShot(SoundHurtObstacle);
        }

        _isDead = true;
        foreach (Collider collider in GetComponentsInChildren<Collider>())
        {
            collider.enabled = false;
        }

        Vector3 headPosition = _playerHead.transform.parent.GetComponent<TargetFollower>().GetPosition();
        _memoTargets[_targetIndex - 1] = headPosition;
        _playerHead.transform.parent.GetComponent<TargetFollower>().FollowTarget(headPosition);

        Camera.main.GetComponent<CameraFollow>().Stop();

        StartCoroutine(WaitUntilQuit());
    }

    IEnumerator WaitUntilQuit()
    {
        // Prevent immediate return to menu
        yield return new WaitForSeconds(1);

        while (true)
        {
            if (Input.GetKeyDown(KeyCode.LeftArrow) || Input.GetKeyDown(KeyCode.RightArrow) || Input.GetKeyDown(KeyCode.Escape))
            {
                break;
            }
            yield return 0;
        }

        SceneManager.LoadScene("SceneMenu");
    }

    public void OnDrawGizmos()
    {
        float size = 0.5f;
        Gizmos.color = Color.red;
        for (int i = _targetIndex - 1; i >= 0 && i >= _targetIndex - 3; i--)
        {
            Gizmos.DrawWireSphere(_memoTargets[i] + new Vector3(0,1,0), size);
            size *= 0.8f;
            Gizmos.color = Color.magenta;
        }
    }
}
